/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoDropDownMap = new Array();
var EchoDropDownBMGs = new Array();

//========================================================================================================
//
// The EchoDropDown object
//
//========================================================================================================
function EchoDropDown(htmlId, outerId, expansionState, alignment, hideHtmlId, boxMutexGroup) {
	// members
	this.htmlId = htmlId;
	this.outerId = outerId;
	this.expansionState = expansionState;
	this.alignment = alignment;
	this.hideHtmlId = null;
	if (hideHtmlId)
		this.hideHtmlId = hideHtmlId;
	this.dropDownWidth = 0;
	this.boxMutexGroup = null;
	
	// EchoDropDownBMGs holds a list of EDD objects that map to a
	// particular BoxMutexGroup name
	if (boxMutexGroup) {
		this.boxMutexGroup = boxMutexGroup;
		if (EchoDropDownBMGs[boxMutexGroup] == null)
			EchoDropDownBMGs[boxMutexGroup] = new Array();
		EchoDropDownBMGs[boxMutexGroup].push(this);
	}

	EchoDropDownMap[htmlId] = this;
}

//---------------------------------------------
// Called to hide/show select fields under the
// hideHtmlId
//---------------------------------------------
EchoDropDown.prototype.toggleSelectFields = function(hideFlag) {
	if (browser.isIE) {
		if (hideFlag)
			epbe_hideSelectFieldsUnder(this.hideHtmlId);
		else
			epbe_showSelectFieldsUnder(this.hideHtmlId);
	}
}

//---------------------------------------------
// Called to position the dropDown 
//---------------------------------------------
EchoDropDown.prototype.positionDropDown = function() {
	if (this.hideHtmlId != null) {
		var eOuter = getElementById(this.outerId);
		var eDropDown = getElementById(this.hideHtmlId);

		// Display first, so that widths are calculated
		if (this.expansionState) {
			eDropDown.setDisplayed(true);
			this.toggleSelectFields(true);
		}

		// We always want to calculate sizes with the dropdown visible
		var x = eOuter.getPageX();
		var y = eOuter.getPageY();
		var w = eOuter.getWidth();
		var h = eOuter.getHeight();
		var ddw = eDropDown.getWidth();
		var ddh = eDropDown.getHeight();
		var wAdjust = ddw - w;
		var scrollX = eOuter.calcScrollX();
		var scrollY = eOuter.calcScrollY();
		
		var newX = 0;
		var newY = 0;
		if (this.expansionState) {
			if (this.alignment == "ALIGN_RIGHT") {
				newX  = x - wAdjust - scrollX;
				newY  = y + h - scrollY;
			} else {
				newX  = x - scrollX;
				newY  = y + h - scrollY;
			}
			eDropDown.moveTo(newX,newY);
		} else {
			eDropDown.setDisplayed(false);
		}
	}
}

//---------------------------------------------
// Called to toggle the state of the DropDown
//---------------------------------------------
EchoDropDown.prototype.toggle = function() {

	// Close other DropDownBox objects in the same BoxMutexGroup
	if (this.boxMutexGroup != null) {
		var boxes = EchoDropDownBMGs[this.boxMutexGroup];
		if (boxes != null) {
			for (var i = 0; i < boxes.length; i++) {
				if (boxes[i] != this && boxes[i].expansionState == true)
					boxes[i].toggle();
			}
		}
	}

	this.expansionState = !this.expansionState;
	this.positionDropDown();
	this.saveState();
}

//---------------------------------------------
// Called to save the state of the DropDown
//---------------------------------------------
EchoDropDown.prototype.saveState = function() {
	var stateStr = '';
	var scrollableE = getElementById(this.htmlId);
	if (scrollableE) {
		stateStr += scrollableE.htmlE.scrollLeft + "," + scrollableE.htmlE.scrollTop + '|';
	}

	if (this.expansionState)
		stateStr += 'true';
	else
		stateStr += 'false';

	//
	// save our client side state
	if (E_setParameter)
		E_setParameter(this.htmlId, stateStr);
}

//---------------------------------------------
// Global function to toggle the time field
//---------------------------------------------
function g_dropdown_toggle(htmlId) {
	var df = EchoDropDownMap[htmlId];
	if (df != null)
		df.toggle();
}

//---------------------------------------------
// Global function to save the state of the drop down
//---------------------------------------------
function g_dropdown_saveState(htmlId) {
	var df = EchoDropDownMap[htmlId];
	if (df != null)
		df.saveState();
}

//---------------------------------------------
// Global function to hide select fields 
// at document onload
//---------------------------------------------
function g_dropdown_onload(htmlId) {
	var df = EchoDropDownMap[htmlId];
	if (df != null)
		df.positionDropDown();
}
