/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
//========================================================================================================
// Constants
//========================================================================================================
var ALIGN_LEFT = 1;
var ALIGN_TOP  = 2;
var ALIGN_RIGHT = 4;
var ALIGN_BOTTOM = 8;
var ALIGN_CENTER = 16;

//========================================================================================================
// Global Functions and Objects
//========================================================================================================

//- - - - - - - - - - - - - - - - - - - - 
// EchoPoint Browser object
//- - - - - - - - - - - - - - - - - - - - 
function EPBrowser() {
	this.agent = navigator.userAgent.toLowerCase();
	this.major = parseInt(navigator.appVersion);
	this.isDOM = (document.getElementById) ? 1 : 0; // true for ie6, ns6, moz 1
	this.isNS = (document.layers);
	this.isNS4up = (this.isNS && this.major >= 4);
	this.isNS6 = (this.isDOM && navigator.appName == "Netscape");
	this.isOpera = this.agent.indexOf('opera') != -1;
	this.isIE = (document.all);
	this.isIE4 = (document.all && !this.isDOM) ? 1 : 0;
	this.isIE4up = (this.isIE && this.major >= 4);
	this.isIE5 = (document.all && this.isDOM);
	this.isWin = ((this.agent.indexOf("win") != -1) || (this.agent.indexOf("16bit") != -1));
	this.isMac = (this.agent.indexOf("mac") != -1);
	this.isGecko = (this.agent.indexOf("Gecko") != -1);
}
var browser = new EPBrowser();

//- - - - - - - - - - - - - - - - - - - - 
// Called to register a function with the top level document
// event handler. The event type is the name, as in "click"
// or "keydown".  The function is the JS function to call
// and codeAraea is used to help debugging and should describe
// the "code reason" eg "combobox" or "datepicker"
//- - - - - - - - - - - - - - - - - - - - 
function epbe_registerDocumentEventHandler(eventType,func, codeArea) {
	g_epbeEventHandlers[g_epbeEventHandlers.length] = new EventHandlerRegistration(eventType,func,codeArea);
}

//- - - - - - - - - - - - - - - - - - - - 
// You create one of this objects and add it to the 
// end of the global array of event handlers (g_epbeEventHandlers)
// like so :
//
// g_epbeEventHandlers[g_epbeEventHandlers.length] = new EventHandlerRegistration("click",func);
//
//- - - - - - - - - - - - - - - - - - - - 
function EventHandlerRegistration(eventType,func,codeArea) {
	this.eventType = eventType;
	this.func = func;
	this.codeArea = codeArea;
}

//- - - - - - - - - - - - - - - - - - - - 
// Global Array of EventHandlers
//- - - - - - - - - - - - - - - - - - - - 
var g_epbeEventHandlers = new Array();

//- - - - - - - - - - - - - - - - - - - - 
// Our top level document event handler
//- - - - - - - - - - - - - - - - - - - - 
function __epbeEventHandler(event) {
	var ev = browser.isIE ? window.event : event;
	for (x in g_epbeEventHandlers) {
		var evh = g_epbeEventHandlers[x];
		if (evh.eventType == ev.type) {
			evh.func(ev);
		}
	}
}

//
// We handle the following document events
document.onclick=__epbeEventHandler;
document.onmouseover=__epbeEventHandler;
document.onkeydown=__epbeEventHandler;
document.onkeyup=__epbeEventHandler;
document.onkeypress=__epbeEventHandler;

//- - - - - - - - - - - - - - - - - - - - 
// SELECT FIELD HIDING
//- - - - - - - - - - - - - - - - - - - - 
var g_epbeExemptSelectFields = new Array();

//--------------------------------------------
// Returns true if the testE inclusively overlaps 
// with the sourceE with an optional 
// toleranceFactor
//--------------------------------------------
function epbe_overlaps(sourceE, testE, toleranceFactor) {
	var tolerance = 0;
	if (toleranceFactor)
		tolerance = toleranceFactor;

	var tw = sourceE.getWidth();
	var th = sourceE.getHeight();
	var ew = testE.getWidth();
	var eh = testE.getHeight();

	if (ew <= 0 || eh <= 0 || tw <= 0 || th <= 0) {
		return false;
	}

	var x1 = sourceE.getPageX();
	var y1 = sourceE.getPageY();
	var x2 = x1 + tw;
	var y2 = y1 + th;

	var ex1 = testE.getPageX();
	var ey1 = testE.getPageY();
	var ex2 = ex1 + ew;
	var ey2 = ey1 + eh;

	if (ex1 - tolerance > x2)
		return false;
	if (ey1 - tolerance > y2)
		return false;
	if (ex2 + tolerance < x1)
		return false;
	if (ey2 + tolerance < y1)
		return false;

	return true;
}

//-------------------------------------------------
// This function "registers" a particular Select field
// as being "exempt" from the standard EPBE
// select field hiding.  You would use this to indicate
// the Select field manages its own state and should not
// be hidden/shown by the epbe_showSelectFieldsUnder()
// or epbe_hideSelectFieldsUnder() methods
//-------------------------------------------------
function epbe_registerExemptSelectField(htmlId) {
	g_epbeExemptSelectFields[htmlId] = htmlId;
}

//-------------------------------------------------
// See epbe_registerExemptSelectField
//-------------------------------------------------
function epbe_isExemptSelectField(htmlId) {
	var id = g_epbeExemptSelectFields[htmlId];
	return id == htmlId;
}

//-------------------------------------------------
// PRIVATE : recursive hides all select fields except
// those that are children of the exceptHtmlId
//-------------------------------------------------
function _hideAllChildSelectFields(parent, hideFlag, exceptHtmlId) {
	var i = 0;
	if (parent.id) {
		if (parent.id == exceptHtmlId) {
			return;
		}
	}
	for (i = 0; i < parent.childNodes.length; i++) {
		var child = parent.childNodes[i];
		if (child.options) {
			if (hideFlag)
				child.style.visibility = 'hidden';
			else
				child.style.visibility = 'visible';
		} else {
			if (child.childNodes.length > 0)
				_hideAllChildSelectFields(child, hideFlag, exceptHtmlId);
		}
	}
}

//-------------------------------------------------
// Hides all select fields that are not children of
// the specified htmlId
//
// You must still do a browser.IsIE check before 
// calling this function.  It is not IE aware.
//
//-------------------------------------------------
function epbe_hideAllSelectFields(exceptHtmlId) {
	if (document.childNodes) {
		_hideAllChildSelectFields(document, true, exceptHtmlId);
	}
}

//-------------------------------------------------
// Hides all select fields that are not children of
// the specified htmlId
//
// You must still do a browser.IsIE check before 
// calling this function.  It is not IE aware.
//
//-------------------------------------------------
function epbe_showAllSelectFields(exceptHtmlId) {
	if (document.childNodes) {
		_hideAllChildSelectFields(document, false, exceptHtmlId);
	}
}

//-------------------------------------------------
// PRIVATE : support funtion to hide/show select fields
// under a given element.  Otherwise all other fields
// are assumed to be showing and if the hideFlag is true
// they are set to shown
//-------------------------------------------------
function _hideChildSelectFieldsUnder(parent, hideFlag, underEArray) {
	var i, j = 0;
	var tolerance = 0;
	if (browser.isIE)
		tolerance = 3;
	if (parent.id) {
		for (j = 0; j < underEArray.length; j++) {
			if (parent.id == underEArray[j].id) {
				return;
			}
		}
	}
	for (i = 0; i < parent.childNodes.length; i++) {
		var child = parent.childNodes[i];
		if (child.options) {
			var overlaps = false;
			var testE = new E(child);
			if (! epbe_isExemptSelectField(testE.id)) {
				for (j = 0; j < underEArray.length; j++) {
					var sourceE = underEArray[j];
					if (epbe_overlaps(sourceE,testE, tolerance)) {
						if (sourceE.id != testE.id) {
							overlaps = true;
							break;
						}
					}
				}
				if (overlaps) {
					if (hideFlag) {
						child.style.visibility = 'hidden';
					} else {
						child.style.visibility = 'visible';
					}
				} else {
					if (hideFlag) {
						child.style.visibility = 'visible';
					}
				}
			}
		} else {
			if (child.childNodes.length > 0)
				_hideChildSelectFieldsUnder(child, hideFlag, underEArray);
		}
	}
}

//-------------------------------------------------
// Hides all select fields that overlap with a given
// set of htmlIds, id1|id2|....|idn
//
// You must still do a browser.IsIE check before 
// calling this function.  It is not IE aware.
//
//-------------------------------------------------
function epbe_hideSelectFieldsUnder(htmlIds) {
	var underEArray = new Array();
	var ids = htmlIds.split('|');
	for (var i = 0; i < ids.length; i++) {
		underEArray[i] = getElementById(ids[i]);
	}
	_hideChildSelectFieldsUnder(document, true, underEArray);
}

//-------------------------------------------------
// Shows all select fields that overlap with a given
// set of htmlIds, id1|id2|....|idn
//
// You must still do a browser.IsIE check before 
// calling this function.  It is not IE aware.
//
//-------------------------------------------------
function epbe_showSelectFieldsUnder(htmlIds) {
	var underEArray = new Array();
	var ids = htmlIds.split('|');
	for (var i = 0; i < ids.length; i++) {
		underEArray[i] = getElementById(ids[i]);
	}
	_hideChildSelectFieldsUnder(document, false, underEArray);
}

