/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//- - - - - - - - - - - - - - - - - - - - 
// Global Mouse Capture object
//- - - - - - - - - - - - - - - - - - - - 

var mouseCapture = new Object();
mouseCapture.reporting = false;
mouseCapture.backgroundImage = null;
mouseCapture.hideSelectFields = true;
mouseCapture.report = epbe_mouseCaptureReport;
mouseCapture.zIndex = 10;

//- - - - - - - - - - - - - - - - - - - - 
function epbe_mouseCaptureReset() {
	mouseCapture.dragE = null; // the element we are dragging
	mouseCapture.target = null; // the actual target element
	mouseCapture.dragWindow = null; // a transparent drag window
	mouseCapture.backgroundImage = null; // an background image for the drag window
	mouseCapture.hideSelectFields = true;
	mouseCapture.zIndex = 0;
	mouseCapture.cursorX = 0;
	mouseCapture.cursorY = 0;
	mouseCapture.cursorStartX = 0;
	mouseCapture.cursorStartY = 0;
	mouseCapture.cursorDeltaX = 0;
	mouseCapture.cursorDeltaY = 0;
	mouseCapture.elemStartLeft = 0;
	mouseCapture.elemStartTop = 0;
	mouseCapture.elemStartWidth = 0;
	mouseCapture.elemStartHeight = 0;
	mouseCapture.dragStartLeft = 0;
	mouseCapture.dragStartTop = 0;
	mouseCapture.dragStartWidth = 0;
	mouseCapture.dragStartHeight = 0;
	
	mouseCapture.onDragFunc = epbe_defaultOnDrag;
	mouseCapture.dragContext = null;
}

//- - - - - - - - - - - - - - - - - - - - 
// Drag reporting function
//
function epbe_mouseCaptureReport(eventType) {
	if (!mouseCapture.reporting)
		return;

	var s = '';
	s += eventType + ':'
	s += ' mx:'+mouseCapture.cursorX;
	s += ' my:'+mouseCapture.cursorY;
	s += ' msx:'+mouseCapture.cursorStartX;
	s += ' msy:'+mouseCapture.cursorStartY;
	s += ' mdx:'+mouseCapture.cursorDeltaX;
	s += ' mdy:'+mouseCapture.cursorDeltaY;
	s += ' esx:'+mouseCapture.elemStartLeft;
	s += ' esy:'+mouseCapture.elemStartTop;
	s += ' dsx:'+mouseCapture.dragStartLeft;
	s += ' dsy:'+mouseCapture.dragStartTop;
	s += ' z:'+mouseCapture.zIndex;
	var e = mouseCapture.dragE;
	if (e != null) {
		s += ' ex:'+e.getX();
		s += ' ey:'+e.getY();
		s += ' epx:'+e.getPageX();
		s += ' epy:'+e.getPageY();
		s += ' id:'+e.id;
		s += ' ew:'+e.getWidth();
		s += ' eh:'+e.getHeight();
	}
	window.status = s;
	return s;
}

//- - - - - - - - - - - - - - - - - - - - 
// Event Prevention
//
function epbe_preventEventBubble(event) {
	if (browser.isIE) {
		window.event.cancelBubble = true;
		window.event.returnValue = false;
	}
	if (browser.isNS6) {
		event.preventDefault();
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// Drag start support function.  Can take a event, and html element id to drag and 
// a custom onDragFunc function to be used on mouse down, mouse move and mouse up events.  
// The drag context object can also be accessed if present.
//
// Ends up calling the onDragFunc function attached to 
// the mouse capture object with an eventType == "mousedown"
// 
// The function signature is :
//
//		yourfunc(mouseCapture,eventType, event);
//
// If the function returns false, then the mouse capture is
// terminated
//
function epbe_dragStart(event, htmlId, onDragFunc, dragContext, useDragWindow) {

	var dragWindowActive = true;
	var e;
	var x, y;

	x = 0;
	y = 0;

	// our custom onMouseMove function
	mouseCapture.onDragFunc = epbe_defaultOnDrag;
	if (onDragFunc) {
		mouseCapture.onDragFunc = onDragFunc;
	}
	mouseCapture.dragContext = null;
	if (dragContext) {
		mouseCapture.dragContext = dragContext;
	}

	if (useDragWindow != null) {
		dragWindowActive = useDragWindow;
	}	
	

	// If an element id was given, find it. Otherwise use the element being
	// clicked on.
	if (htmlId) {
		e = getElementById(htmlId);
	} else {
		if (browser.isIE)
			e = new E(window.event.srcElement);
		if (browser.isNS6)
			e = new E(event.target);

		// If this is a text node, use its parent element.
		if (e.htmlE.nodeType == 3)
			e = new E(e.htmlE.parentNode);
	}
	mouseCapture.dragWindow = null;
	mouseCapture.target = e;
	mouseCapture.dragE = e;
	
	mouseCapture.pageScrollX = 0;
	mouseCapture.pageScrollY = 0;

	// Get cursor position with respect to the page.
	if (browser.isIE) {
		mouseCapture.pageScrollX = document.documentElement.scrollLeft + document.body.scrollLeft;
		mouseCapture.pageScrollY = document.documentElement.scrollLeft + document.body.scrollLeft;
		x = window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft;
		y = window.event.clientY + document.documentElement.scrollTop + document.body.scrollTop;
	}
	if (browser.isNS6) {
		mouseCapture.pageScrollX = window.scrollX;
		mouseCapture.pageScrollY = window.scrollY;
		x = event.clientX + window.scrollX;
		y = event.clientY + window.scrollY;
	}
	// Save starting positions of cursor and element.
	mouseCapture.cursorX = x;
	mouseCapture.cursorY = y;
	mouseCapture.cursorStartX = x;
	mouseCapture.cursorStartY = y;
	mouseCapture.cursorDeltaX = 0;
	mouseCapture.cursorDeltaY = 0;
	
	if (e.isAbsolute()) {
		mouseCapture.elemStartLeft   = parseInt(e.getX(), 10);
		mouseCapture.elemStartTop    = parseInt(e.getY(), 10);
	} else {
		mouseCapture.elemStartLeft   = parseInt(e.htmlE.style.left, 10);
		mouseCapture.elemStartTop    = parseInt(e.htmlE.style.top, 10);
	}
	mouseCapture.elemStartWidth  = parseInt(e.getWidth(), 10);
	mouseCapture.elemStartHeight = parseInt(e.getHeight(), 10);

	if (isNaN(mouseCapture.elemStartLeft))		mouseCapture.elemStartLeft = 0;
	if (isNaN(mouseCapture.elemStartTop))  		mouseCapture.elemStartTop = 0;
	if (isNaN(mouseCapture.elemStartWidth)) 	mouseCapture.elemStartWidth = 0;
	if (isNaN(mouseCapture.elemStartHeight))	mouseCapture.elemStartHeight = 0;
	
	mouseCapture.dragStartLeft   = mouseCapture.elemStartLeft;
	mouseCapture.dragStartTop    = mouseCapture.elemStartTop;
	mouseCapture.dragStartWidth  = mouseCapture.elemStartWidth;
	mouseCapture.dragStartHeight = mouseCapture.elemStartHeight;
	
	// use a drag window if we can create it
	if (document.appendChild && dragWindowActive) {
		var draggerE = _epbe_createDragWindow(e);
		mouseCapture.dragWindow = draggerE
		mouseCapture.dragE = draggerE;
		
		mouseCapture.dragStartLeft   = parseInt(draggerE.getX(), 10);
		mouseCapture.dragStartTop    = parseInt(draggerE.getY(), 10);
		mouseCapture.dragStartWidth  = parseInt(draggerE.getWidth(), 10);
		mouseCapture.dragStartHeight = parseInt(draggerE.getHeight(), 10);
		
	}
	// z-index
	mouseCapture.target.setZ(++mouseCapture.zIndex);
	mouseCapture.dragE.setZ(mouseCapture.zIndex);

	// now call our on mouse down function to let them to custom processing
	// if they return false then jump out.
	// call their custom function if present  	
	if (mouseCapture.onDragFunc != null) {
		var rc = mouseCapture.onDragFunc(mouseCapture, "mousedown", event);
		if (rc == false) {
			epbe_preventEventBubble(event);
			return;
		}
	}

	// Capture mousemove and mouseup events on the page to our functions.
	if (browser.isIE) {
		document.attachEvent("onmousemove", epbe_dragGo);
		document.attachEvent("onmouseup", epbe_dragStop);
	}
	if (browser.isNS6) {
		document.addEventListener("mousemove", epbe_dragGo, true);
		document.addEventListener("mouseup", epbe_dragStop, true);
	}
	// stop the event		
	epbe_preventEventBubble(event);
}

//- - - - - - - - - - - - - - - - - - - - 
// Drag go support function.  
//
// Ends up calling the onDragFunc function attached to 
// the mouse capture object with an eventType == "mousemove"
// 
// The function signature is :
//
//		yourfunc(mouseCapture,eventType, event);
//
// If the function returns false, then the mouse capture is
// terminated
//
function epbe_dragGo(event) {

	var x, y;
	// Get cursor position with respect to the page.
	if (browser.isIE) {
		x = window.event.clientX + document.documentElement.scrollLeft + document.body.scrollLeft;
		y = window.event.clientY + document.documentElement.scrollTop + document.body.scrollTop;
	}
	if (browser.isNS6) {
		x = event.clientX + window.scrollX;
		y = event.clientY + window.scrollY;
	}
	mouseCapture.cursorX = x;
	mouseCapture.cursorY = y;
	mouseCapture.cursorDeltaX = x - mouseCapture.cursorStartX;
	mouseCapture.cursorDeltaY = y - mouseCapture.cursorStartY;

	// call the attached function
	if (mouseCapture.onDragFunc != null) {
		var rc = mouseCapture.onDragFunc(mouseCapture, "mousemove", event);
		if (rc == false) {
			epbe_preventEventBubble(event);
			return;
		}
	}

	// hide any select fields
	if (browser.isIE && mouseCapture.hideSelectFields && mouseCapture.target.id) {
		epbe_hideSelectFieldsUnder(mouseCapture.target.id);
	}

	// stop the event		
	epbe_preventEventBubble(event);
}

//- - - - - - - - - - - - - - - - - - - - 
// Drag stop support function
//
// Ends up calling the onDragFunc function attached to 
// the mouse capture object with an eventType == "mouseup"
// 
// The function signature is :
//
//		yourfunc(mouseCapture,eventType, event);
//
// If the function returns false, then the mouse capture is
// terminated
//
function epbe_dragStop(event) {

	// Stop capturing mousemove and mouseup events.

	if (browser.isIE) {
		document.detachEvent("onmousemove", epbe_dragGo);
		document.detachEvent("onmouseup", epbe_dragStop);
	}
	if (browser.isNS6) {
		document.removeEventListener("mousemove", epbe_dragGo, true);
		document.removeEventListener("mouseup", epbe_dragStop, true);
	}
	// swap back to actual target
	mouseCapture.dragE = mouseCapture.target;
	if (mouseCapture.dragWindow != null) {
		document.body.removeChild(mouseCapture.dragWindow.htmlE);
	}

	if (mouseCapture.onDragFunc != null) {
		var rc = mouseCapture.onDragFunc(mouseCapture, "mouseup", event);
	}

	// hide any select fields
	if (browser.isIE && mouseCapture.hideSelectFields && mouseCapture.target.id) {
		epbe_hideSelectFieldsUnder(mouseCapture.target.id);
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// Default onDrag function.  Moves the element
// to the specified co-ordinates during the
// "mousemove" event type.  The "mouseup" and
// "mousedown" event types do nothing.
//
// In general you will want to call this 
// function to provide default dragging behavior
//
function epbe_defaultOnDrag(mouseCapture, eventType, event) {
	if (mouseCapture.reporting)
		mouseCapture.report(eventType);

	var e = mouseCapture.dragE;

	if (eventType == "mousemove") {
		var newX = mouseCapture.dragStartLeft + mouseCapture.cursorDeltaX;
		var newY = mouseCapture.dragStartTop + mouseCapture.cursorDeltaY;
		e.setX(newX);
		e.setY(newY);

	} else if (eventType == "mouseup") {		
		var newX = mouseCapture.elemStartLeft + mouseCapture.cursorDeltaX;
		var newY = mouseCapture.elemStartTop + mouseCapture.cursorDeltaY;
		e.setX(newX);
		e.setY(newY);
		
	} else if (eventType == "mousedown") {
		;
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// onDrag function that resizes the
// width of the dragE element in question 
// by deltaX contrained by 0
//
function epbe_resizeXOnDrag(mouseCapture, eventType, event) {
	if (mouseCapture.reporting)
		mouseCapture.report(eventType);

	if (eventType == "mousemove" || eventType == "mouseup") {
		var e = mouseCapture.dragE;
		var w = mouseCapture.elemStartWidth + mouseCapture.cursorDeltaX;
		var amount = Math.max(w, 0);
		e.setWidth(amount);
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// onDrag function that resizes the
// height of the dragE element in question 
// by deltaY contrained by 0
//
function epbe_resizeYOnDrag(mouseCapture, eventType, event) {
	if (mouseCapture.reporting)
		mouseCapture.report(eventType);

	if (eventType == "mousemove" || eventType == "mouseup") {
		var e = mouseCapture.dragE;
		var h = mouseCapture.elemStartHeight + mouseCapture.cursorDeltaY;
		var amount = Math.max(h, 0);
		e.setHeight(amount);
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// onDrag function that resizes the
// width and height of the dragE element in question 
// by deltaX, delatY contrained by 0,0
//
function epbe_resizeOnDrag(mouseCapture, eventType, event) {
	epbe_resizeXOnDrag(mouseCapture, eventType, event);
	epbe_resizeYOnDrag(mouseCapture, eventType, event);
}

//-------------------------------------------------
// PRIVATE : creates a transparent dragWindow presized
//-------------------------------------------------
function _epbe_createDragWindow(target) {
	var dragWindow = document.createElement('DIV');

	dragWindow.style.position = "absolute";
	dragWindow.style.zIndex = 32000;
	dragWindow.style.width = target.getWidth()  + "px";
	dragWindow.style.height = target.getHeight()+ "px";
	
	var newX = target.getPageX();
	var newY = target.getPageY();
	var parentE = target.getParent();
	if (parentE && target.isRelative()) {
		newX -= parentE.getScrollX();
		newY -= parentE.getScrollY();
	}
	dragWindow.style.left = newX + "px";
	dragWindow.style.top = newY + "px";

	dragWindow.style.cursor = 'move';
	dragWindow.style.borderWidth = 3;
	dragWindow.style.borderStyle = 'dashed';
	if (browser.isIE) {
		dragWindow.style.filter = "alpha(opacity=40)";
		dragWindow.style.backgroundColor = '#666666';
		dragWindow.style.borderStyle = 'solid';
		if (mouseCapture.backgroundImage)
			dragWindow.style.backgroundImage = mouseCapture.backgroundImage;
	} else {
		dragWindow.style.backgroundImage = mouseCapture.backgroundImage;
	}

	var borderColor = '#6A6A6A';
	if (document.body.style && document.body.style.backgroundColor) {
		var bodyBG = epbe_normalizeColorHex(document.body.style.backgroundColor);
		if (bodyBG == borderColor || bodyBG == '#000000')
			borderColor = '#FFFFFF';
	}
	dragWindow.style.borderColor = borderColor; 
	document.body.appendChild(dragWindow);
	return new E(dragWindow);
}
