/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.soafaces.services.client.rpc;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import java.io.Serializable;
import java.util.Map;

/**
 *
 * @author Sam Taha
 */
 class UniversalClientProxy implements UniversalClient {

  private UniversalClientRPCAsync _oUniversalClientService = null;
  
  private UniversalClientProxy() {}

  UniversalClientProxy(String uri)
  {
    _oUniversalClientService = (UniversalClientRPCAsync) GWT.create(UniversalClientRPC.class);

    /*
    GWT.setUncaughtExceptionHandler(new GWT.UncaughtExceptionHandler() {
      public void onUncaughtException(Throwable e) {
        Window.alert("Uncaught Exception: " + e.getMessage());
      }
    });*/

    ServiceDefTarget endpointUniversalClient = (ServiceDefTarget) _oUniversalClientService;
    //String muleModuleRelativeURL = "/" + _stContextName + "/rpcservices/UniversalClientservice?tasktype=" +_stTaskType;
    String muleModuleRelativeURL = uri;
    endpointUniversalClient.setServiceEntryPoint(muleModuleRelativeURL);

  }
  
  public void send(String endpointURI, Object payload, Map<String,Serializable> properties, UniversalClientPOJOCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oUniversalClientService.sendAnyReturnPOJO(endpointURI, serializablePayload, properties, callback);
  }

  public void send(String endpointURI, Object payload, UniversalClientPOJOCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oUniversalClientService.sendAnyReturnPOJO(endpointURI, serializablePayload, null, callback);
  }
   
  public void send(String endpointURI, UniversalClientPOJOCallback callback) {
    _oUniversalClientService.sendEmptyReturnPOJO(endpointURI, null, callback);
  }

  public void send(String endpointURI, Object payload, Map<String,Serializable> properties, UniversalClientJSONCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oUniversalClientService.sendAnyReturnJSON(endpointURI, serializablePayload, properties, callback);
  }

  public void send(String endpointURI, Object payload, UniversalClientJSONCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oUniversalClientService.sendAnyReturnJSON(endpointURI, serializablePayload, null, callback);
  }
  
  public void send(String endpointURI, UniversalClientJSONCallback callback) {
    _oUniversalClientService.sendEmptyReturnJSON(endpointURI, null, callback);
  }

  public void dispatch(String endpointURI, Object payload, Map<String,Serializable> properties, AsyncCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oUniversalClientService.dispatchAny(endpointURI, serializablePayload, properties, callback);
  }

  public void dispatch(String endpointURI, Object payload, AsyncCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oUniversalClientService.dispatchAny(endpointURI, serializablePayload, null, callback);
  }

  public void dispatch(String endpointURI, AsyncCallback callback) {
    _oUniversalClientService.dispatchEmpty(endpointURI, null, callback);
  }
}
